@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-11-29 00:56:25 jcs>
+/* Time-stamp: <2006-09-23 21:25:40 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -30,7 +30,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: itdb.h,v 1.24 2005/11/28 16:20:40 jcsjcs Exp $
+|  $Id: itdb.h,v 1.43 2006/09/23 12:28:23 jcsjcs Exp $
 */
 
 #ifndef __ITUNESDB_H__
@@ -40,8 +40,6 @@
 #  include <config.h>
 #endif
 
-#include "ipod-device.h"
-
 #include <sys/types.h>
 #include <time.h>
 #include <glib.h>
@@ -56,14 +54,18 @@ G_BEGIN_DECLS
 typedef void (* ItdbUserDataDestroyFunc) (gpointer userdata);
 typedef gpointer (* ItdbUserDataDuplicateFunc) (gpointer userdata);
 
+typedef struct _Itdb_Device Itdb_Device;
 typedef struct _Itdb_Artwork Itdb_Artwork;
 typedef struct _Itdb_Thumb Itdb_Thumb;
 typedef struct _SPLPref SPLPref;
 typedef struct _SPLRule SPLRule;
 typedef struct _SPLRules SPLRules;
 typedef struct _Itdb_iTunesDB Itdb_iTunesDB;
+typedef struct _Itdb_PhotoDB Itdb_PhotoDB;
 typedef struct _Itdb_Playlist Itdb_Playlist;
+typedef struct _Itdb_PhotoAlbum Itdb_PhotoAlbum;
 typedef struct _Itdb_Track Itdb_Track;
+typedef struct _Itdb_IpodInfo Itdb_IpodInfo;
 
 
 /* ------------------------------------------------------------ *\
@@ -86,7 +88,7 @@ typedef enum {
 /* The Itdb_Thumb structure can represent two slightly different
    thumbnails:
 
-   - a thumbnail before it's transferred to the iPod.
+  a) a thumbnail before it's transferred to the iPod.
 
      offset and size are 0
 
@@ -96,26 +98,101 @@ typedef enum {
 
      type is set according to the type this thumbnail represents
 
-     filename point to a 'real' image file.
+     filename point to a 'real' image file OR image_data and
+     image_data_len are set.
  
-   - a thumbnail (big or small) stored on a database in the iPod.  In
+  b) a thumbnail (big or small) stored on a database in the iPod.  In
      these cases, id corresponds to the ID originally used in the
      database, filename points to a .ithmb file on the iPod
  */
 struct _Itdb_Thumb {
     ItdbThumbType type;
     gchar *filename;
+    guchar *image_data;      /* holds the thumbnail data of
+				non-transfered thumbnails when
+				filename == NULL */
+    gsize  image_data_len;   /* length of data */
     guint32 offset;
     guint32 size;
     gint16 width;
     gint16 height;
+    gint16 horizontal_padding;
+    gint16 vertical_padding;
 };
 
 struct _Itdb_Artwork {
     GList *thumbnails;    /* list of Itdb_Thumbs */
     guint32 artwork_size; /* Size in bytes of the original source image */
-    guint id;             /* some kind of ID, starting with
+    guint32 id;           /* Artwork id used by photoalbums, starts at
 			   * 0x40... libgpod will set this on sync. */
+    gint32 creation_date; /* Date the image was created */
+    /* below is for use by application */
+    guint64 usertype;
+    gpointer userdata;
+    /* function called to duplicate userdata */
+    ItdbUserDataDuplicateFunc userdata_duplicate;
+    /* function called to free userdata */
+    ItdbUserDataDestroyFunc userdata_destroy;
+};
+
+
+/* ------------------------------------------------------------ *\
+ *
+ * iPod model-relevant definitions
+ *
+\* ------------------------------------------------------------ */
+
+typedef enum {
+    ITDB_IPOD_GENERATION_UNKNOWN,
+    ITDB_IPOD_GENERATION_FIRST,
+    ITDB_IPOD_GENERATION_SECOND,
+    ITDB_IPOD_GENERATION_THIRD,
+    ITDB_IPOD_GENERATION_FOURTH,
+    ITDB_IPOD_GENERATION_FIFTH,
+    ITDB_IPOD_GENERATION_SIXTH,
+    ITDB_IPOD_GENERATION_MOBILE
+} Itdb_IpodGeneration;
+
+typedef enum {
+    ITDB_IPOD_MODEL_INVALID,
+    ITDB_IPOD_MODEL_UNKNOWN,
+    ITDB_IPOD_MODEL_COLOR,
+    ITDB_IPOD_MODEL_COLOR_U2,
+    ITDB_IPOD_MODEL_REGULAR,
+    ITDB_IPOD_MODEL_REGULAR_U2,
+    ITDB_IPOD_MODEL_MINI,
+    ITDB_IPOD_MODEL_MINI_BLUE,
+    ITDB_IPOD_MODEL_MINI_PINK,
+    ITDB_IPOD_MODEL_MINI_GREEN,
+    ITDB_IPOD_MODEL_MINI_GOLD,
+    ITDB_IPOD_MODEL_SHUFFLE,
+    ITDB_IPOD_MODEL_NANO_WHITE,
+    ITDB_IPOD_MODEL_NANO_BLACK,
+    ITDB_IPOD_MODEL_VIDEO_WHITE,
+    ITDB_IPOD_MODEL_VIDEO_BLACK,
+    ITDB_IPOD_MODEL_MOBILE_1,
+    ITDB_IPOD_MODEL_VIDEO_U2,
+    ITDB_IPOD_MODEL_NANO_SILVER,
+    ITDB_IPOD_MODEL_NANO_BLUE,
+    ITDB_IPOD_MODEL_NANO_GREEN,
+    ITDB_IPOD_MODEL_NANO_PINK
+} Itdb_IpodModel;
+
+struct _Itdb_IpodInfo {
+       /* model_number is abbreviated: if the first character is not
+	  numeric, it is ommited. e.g. "MA350 -> A350", "M9829 -> 9829" */
+	const gchar *model_number;
+        const double capacity;  /* in GB */
+	const Itdb_IpodModel ipod_model;
+	const Itdb_IpodGeneration ipod_generation;
+        /* Number of music (Fnn) dirs created by iTunes. The exact
+	   number seems to be version dependent. Therefore, the
+	   numbers here represent a mixture of reported values and
+	   common sense. Note: this number does not necessarily
+	   represent the number of dirs present on a particular
+	   iPod. It is used when setting up a new iPod from
+	   scratch. */
+        const guint musicdirs;
 };
 
 
@@ -389,14 +466,26 @@ struct _SPLRules
 /* one star is how much (track->rating) */
 #define ITDB_RATING_STEP 20
 
+struct _Itdb_PhotoDB
+{
+    GList *photos;
+    GList *photoalbums;
+    Itdb_Device *device;/* iPod device info     */
+    /* below is for use by application */
+    guint64 usertype;
+    gpointer userdata;
+    /* function called to duplicate userdata */
+    ItdbUserDataDuplicateFunc userdata_duplicate;
+    /* function called to free userdata */
+    ItdbUserDataDestroyFunc userdata_destroy;
+};
+
 struct _Itdb_iTunesDB
 {
     GList *tracks;
     GList *playlists;
     gchar *filename;    /* filename of iTunesDB */
-    gchar *mountpoint;  /* mountpoint of iPod (if available) */
-    IpodDevice *device;
-    gint   musicdirs;   /* number of /iPod_Control/Music/F.. dirs */
+    Itdb_Device *device;/* iPod device info     */
     guint32 version;
     guint64 id;
     /* below is for use by application */
@@ -408,12 +497,31 @@ struct _Itdb_iTunesDB
     ItdbUserDataDestroyFunc userdata_destroy;
 };
 
+struct _Itdb_PhotoAlbum
+{
+    gchar *name;          /* name of photoalbum in UTF8            */
+    GList *members;       /* photos in album (Track *)             */
+    gint  num_images;     /* number of photos in album             */
+    gint  master;         /* 0x01 for master, 0x00 otherwise       */
+    gint  album_id;
+    gint  prev_album_id;
+    /* below is for use by application */
+    guint64 usertype;
+    gpointer userdata;
+    /* function called to duplicate userdata */
+    ItdbUserDataDuplicateFunc userdata_duplicate;
+    /* function called to free userdata */
+    ItdbUserDataDestroyFunc userdata_destroy;
+};
 
 struct _Itdb_Playlist
 {
     Itdb_iTunesDB *itdb;  /* pointer to iTunesDB (for convenience) */
     gchar *name;          /* name of playlist in UTF8              */
-    guint32 type;         /* ITDB_PL_TYPE_NORM/_MPL                */
+    guint8 type;          /* ITDB_PL_TYPE_NORM/_MPL                */
+    guint8 flag1;         /* unknown, usually set to 0             */
+    guint8 flag2;         /* unknown, always set to 0              */
+    guint8 flag3;         /* unknown, always set to 0              */
     gint  num;            /* number of tracks in playlist          */
     GList *members;       /* tracks in playlist (Track *)          */
     gboolean is_spl;      /* smart playlist?                       */
@@ -547,6 +655,9 @@ struct _Itdb_Track
   gint32  tracks;            /* number of tracks       */
   gint32  bitrate;           /* bitrate                */
   guint16 samplerate;        /* samplerate (CD: 44100) */
+  guint16 samplerate_low;    /* in the iTunesDB the samplerate is
+                                multiplied by 0x10000 -- these are the
+				lower 16 bit, which are usually 0 */
   gint32  year;              /* year                   */
   gint32  volume;            /* volume adjustment              */
   guint32 soundcheck;        /* volume adjustment "soundcheck" */
@@ -563,33 +674,35 @@ struct _Itdb_Track
 				playcount. */
   guint32 recent_playcount;  /* times track was played since last sync */
   gboolean transferred;      /* has file been transferred to iPod?  */
-  gint16 BPM;                /* supposed to vary the playback speed */
+  gint16  BPM;               /* supposed to vary the playback speed */
   guint8  app_rating;        /* star rating set by appl. (not
 			      * iPod). If the rating set on the iPod
 			        and the rating field above differ, the
 				original rating is copied here and the
 				new rating is stored above. */
-  guint16 type;              /* CBR MP3s are type 0x100, VBR MP3s are
-			        type 0x101, AAC are type 0x0 */
+  guint8  type1;             /* CBR MP3s and AAC are 0x00, VBR MP3s are
+			        0x01 */
+  guint8  type2;             /* MP3s are 0x01, AAC are 0x00 */
   guint8  compilation;
   guint32 starttime;
   guint32 stoptime;
-  guint8  checked;
+  guint8  checked;           /* 0x0: checkmark on track is set 0x1: not set */
   guint64 dbid;              /* unique database ID */
   guint32 drm_userid;        /* Apple Store/Audible User ID (for DRM'ed
 				files only, set to 0 otherwise). */
   guint32 visible;           /*  If this value is 1, the song is visible
 				 on the iPod. All other values cause
 				 the file to be hidden. */
-  gchar filetype_marker[4];  /* This appears to always be 0 on hard
+  guint32 filetype_marker;   /* This appears to always be 0 on hard
                                 drive based iPods, but for the
                                 iTunesDB that is written to an iPod
                                 Shuffle, iTunes 4.7.1 writes out the
                                 file's type as an ANSI string(!). For
                                 example, a MP3 file has a filetype of
                                 0x4d503320 -> 0x4d = 'M', 0x50 = 'P',
-                                0x33 = '3', 0x20 = <space>. (always
-                                left set to 0 by itdb)*/
+                                0x33 = '3', 0x20 = <space>. (set to
+				the filename extension by itdb when
+				copying track to iPod)*/
   guint16 artwork_count;     /* The number of album artwork items
 				associated with this song. libgpod
 				updates this value when syncing */
@@ -605,7 +718,6 @@ struct _Itdb_Track
 				here.  itdb will set this when adding
 				a track */
 
-  guint16 unk060;     /* unknown */
   guint16 unk126;     /* unknown, but always seems to be 0xffff for
 			 MP3/AAC songs, 0x0 for uncompressed songs
 			 (like WAVE format), 0x1 for Audible. itdb
@@ -616,25 +728,28 @@ struct _Itdb_Track
 			 0700 GMT. For podcasts: release date as
 			 displayed next to the title in the Podcast
 			 playlist  */
-  guint32 unk144;     /* unknown, but MP3 songs appear to be always
-			 0x0000000c or 0x0100000c (if played one or
-			 more times in iTunes), AAC songs are always
-			 0x01000033, Audible files are 0x01000029, WAV
-			 files are 0x0. itdb will attempt to set this
-			 value when adding a track. */  
+  guint16 unk144;     /* unknown, but MP3 songs appear to be always
+			 0x000c, AAC songs are always 0x0033, Audible
+			 files are 0x0029, WAV files are 0x0. itdb
+			 will attempt to set this value when adding a
+			 track. */  
+  guint16 unk146;     /* unknown, but appears to be 1 if played at
+			 least once in iTunes and 0 otherwise. */
   guint32 unk148;     /* unknown - used for Apple Store DRM songs
 			 (always 0x01010100?), zero otherwise */
   guint32 unk152;     /* unknown */
   guint32 unk156, unk160;
-  guint8 flag1;       /* basically unknown. set to 0x02 for
-		       * podcasts, 0x00 otherwise. */
-  guint8 flag2;       /* "Skip when shuffling" when set to 0x01, set
-			 to 0x00 otherwise. .m4b and .aa files always
-			 seem to be skipped when shuffling, however */
-  guint8 flag3;       /* "Remember playback position" when set to
+  guint8 has_artwork; /* 0x01: artwork is present. 0x02: no artwork is
+			 present for this track (used by the iPod to
+			 decide whether to display Artwork or not) */
+  guint8 skip_when_shuffling;/* "Skip when shuffling" when set to
 			 0x01, set to 0x00 otherwise. .m4b and .aa
-			 files always seem to remember the playback
-			 position, however. */
+			 files always seem to be skipped when
+			 shuffling, however */
+  guint8 remember_playback_position;/* "Remember playback position"
+			 when set to 0x01, set to 0x00 otherwise. .m4b
+			 and .aa files always seem to remember the
+			 playback position, however. */
   guint8 flag4;       /* Used for podcasts, 0x00 otherwise.  If set to
 			 0x01 the "Now Playing" page will show
 			 Title/Album, when set to 0x00 it will also
@@ -643,10 +758,14 @@ struct _Itdb_Track
 			 about the track will be shown. */
   guint64 dbid2;      /* not clear. if not set, itdb will set this to
 			 the same value as dbid when adding a track */
-  guint32 unk176;     /* unknown - added in dbversion 0x0c, first
-			 values observed in 0x0d. Observed to be
-			 0x00010000 for non-podcasts and 0x00020000
-			 for a podcast. */
+  guint8 lyrics_flag; /* set to 0x01 if lyrics are present in MP3 tag
+			 ("ULST"), 0x00 otherwise */
+  guint8 movie_flag;  /* set to 0x01 if it's a movie file, 0x00
+		         otherwise */
+  guint8 mark_unplayed; /* A value of 0x02 marks a podcast as unplayed
+			   on the iPod (bullet) once played it is set to
+			   0x01. Non-podcasts have this set to 0x01. */
+  guint8 unk179;      /* unknown (always 0x00 so far) */
   guint32 unk180, unk184;
   guint32 samplecount;/* Number of samples in the song. First observed
 			 in dbversion 0x0d, and only for AAC and WAV
@@ -710,7 +829,7 @@ GQuark     itdb_file_error_quark      (void);
 /* functions for reading/writing database, general itdb functions */
 Itdb_iTunesDB *itdb_parse (const gchar *mp, GError **error);
 Itdb_iTunesDB *itdb_parse_file (const gchar *filename, GError **error);
-gboolean itdb_write (Itdb_iTunesDB *itdb, GError **error);
+gboolean itdb_write (Itdb_iTunesDB *db, GError **error);
 gboolean itdb_write_file (Itdb_iTunesDB *itdb, const gchar *filename,
 			  GError **error);
 gboolean itdb_shuffle_write (Itdb_iTunesDB *itdb, GError **error);
@@ -725,8 +844,8 @@ guint32 itdb_playlists_number (Itdb_iTunesDB *itdb);
 
 /* general file functions */
 gint itdb_musicdirs_number (Itdb_iTunesDB *itdb);
-gchar * itdb_resolve_path (const gchar *root,
-			   const gchar * const * components);
+gchar *itdb_resolve_path (const gchar *root,
+			  const gchar * const * components);
 gboolean itdb_rename_files (const gchar *mp, GError **error);
 gboolean itdb_cp_track_to_ipod (Itdb_Track *track,
 				gchar *filename, GError **error);
@@ -736,6 +855,33 @@ void itdb_filename_fs2ipod (gchar *filename);
 void itdb_filename_ipod2fs (gchar *ipod_file);
 gchar *itdb_filename_on_ipod (Itdb_Track *track);
 void itdb_set_mountpoint (Itdb_iTunesDB *itdb, const gchar *mp);
+const gchar *itdb_get_mountpoint (Itdb_iTunesDB *itdb);
+gchar *itdb_get_control_dir (const gchar *mountpoint);
+gchar *itdb_get_itunes_dir (const gchar *mountpoint);
+gchar *itdb_get_music_dir (const gchar *mountpoint);
+gchar *itdb_get_artwork_dir (const gchar *mountpoint);
+gchar *itdb_get_photos_dir (const gchar *mountpoint);
+gchar *itdb_get_photos_thumb_dir (const gchar *mountpoint);
+gchar *itdb_get_device_dir (const gchar *mountpoint);
+gchar *itdb_get_itunesdb_path (const gchar *mountpoint);
+gchar *itdb_get_itunessd_path (const gchar *mountpoint);
+gchar *itdb_get_artworkdb_path (const gchar *mountpoint);
+gchar *itdb_get_photodb_path (const gchar *mountpoint);
+gchar *itdb_get_path (const gchar *dir, const gchar *file);
+
+/* itdb_device functions */
+Itdb_Device *itdb_device_new (void);
+void itdb_device_free (Itdb_Device *device);
+void itdb_device_set_mountpoint (Itdb_Device *device, const gchar *mp);
+gboolean itdb_device_read_sysinfo (Itdb_Device *device);
+gboolean itdb_device_write_sysinfo (Itdb_Device *device, GError **error);
+gchar *itdb_device_get_sysinfo (Itdb_Device *device, const gchar *field);
+void itdb_device_set_sysinfo (Itdb_Device *device,
+			      const gchar *field, const gchar *value);
+const Itdb_IpodInfo *itdb_device_get_ipod_info (Itdb_Device *device);
+const Itdb_IpodInfo *itdb_info_get_ipod_info_table (void);
+const gchar *itdb_info_get_ipod_model_name_string (Itdb_IpodModel model);
+const gchar *itdb_info_get_ipod_generation_string (Itdb_IpodGeneration generation);
 
 /* track functions */
 Itdb_Track *itdb_track_new (void);
@@ -797,7 +943,29 @@ void itdb_spl_update_live (Itdb_iTunesDB *itdb);
 /* itdb_track_... */
 gboolean itdb_track_set_thumbnails (Itdb_Track *track,
 				    const gchar *filename);
+gboolean itdb_track_set_thumbnails_from_data (Itdb_Track *track,
+					      const guchar *image_data,
+					      gsize image_data_len);
 void itdb_track_remove_thumbnails (Itdb_Track *track);
+
+/* photoalbum functions */
+Itdb_PhotoDB *itdb_photodb_parse (const gchar *mp, GError **error);
+gboolean itdb_photodb_add_photo (Itdb_PhotoDB *db,
+				 const gchar *albumname,
+				 const gchar *filename);
+gboolean itdb_photodb_add_photo_from_data (Itdb_PhotoDB *db,
+					   const gchar *albumname,
+					   const guchar *image_data,
+					   gsize image_data_len);
+Itdb_PhotoAlbum *itdb_photodb_photoalbum_new (Itdb_PhotoDB *db,
+					      const gchar *album_name);
+Itdb_PhotoDB *itdb_photodb_new (void);
+void itdb_photodb_free (Itdb_PhotoDB *photodb);
+gboolean itdb_photodb_write (Itdb_PhotoDB *db, GError **error);
+void itdb_photodb_photoalbum_free (Itdb_PhotoAlbum *pa);
+gboolean itdb_photodb_remove_photo (Itdb_PhotoDB *db,
+        const gint photo_id );
+
 /* itdb_artwork_... */
 Itdb_Artwork *itdb_artwork_new (void);
 Itdb_Artwork *itdb_artwork_duplicate (Itdb_Artwork *artwork);
@@ -807,25 +975,34 @@ Itdb_Thumb *itdb_artwork_get_thumb_by_type (Itdb_Artwork *artwork,
 gboolean itdb_artwork_add_thumbnail (Itdb_Artwork *artwork,
 				     ItdbThumbType type,
 				     const gchar *filename);
+gboolean itdb_artwork_add_thumbnail_from_data (Itdb_Artwork *artwork,
+					       ItdbThumbType type,
+					       const guchar *image_data,
+					       gsize image_data_len);
 void itdb_artwork_remove_thumbnail (Itdb_Artwork *artwork,
 				    Itdb_Thumb *thumb);
 void itdb_artwork_remove_thumbnails (Itdb_Artwork *artwork);
 /* itdb_thumb_... */
 /* the following funciton returns a pointer to a GdkPixbuf if
    gdk-pixbuf is installed -- a NULL pointer otherwise. */
-gpointer itdb_thumb_get_gdk_pixbuf (IpodDevice *device,
+gpointer itdb_thumb_get_gdk_pixbuf (Itdb_Device *device,
 				    Itdb_Thumb *thumb);
 Itdb_Thumb *itdb_thumb_duplicate (Itdb_Thumb *thumb);
 void itdb_thumb_free (Itdb_Thumb *thumb);
 Itdb_Thumb *itdb_thumb_new (void);
-gchar *itdb_thumb_get_filename (IpodDevice *device, Itdb_Thumb *thumb);
-
+gchar *itdb_thumb_get_filename (Itdb_Device *device, Itdb_Thumb *thumb);
 
 /* time functions */
 guint64 itdb_time_get_mac_time (void);
 time_t itdb_time_mac_to_host (guint64 mactime);
 guint64 itdb_time_host_to_mac (time_t time);
 
+/* Initialise a blank ipod */
+gboolean itdb_init_ipod (const gchar *mountpoint,
+			 const gchar *model_number,
+			 const gchar *ipod_name,
+			 GError **error);
+
 G_END_DECLS
 
 #endif
